part of nueli.core.bucket;

Gallery get gallery => Gallery();

class Gallery extends Bucket<Picture> {
  @override
  Symbol get iD => #gallery;
}

class Picture {
  final IconData? data;
  final String? src;
  final String? path;
  static Picture get error => Picture.library(CupertinoIcons.exclamationmark_circle);

  /// Define a new Picture with a reference to an
  /// [Icons] or [CupertinoIcons] data.
  /// ```
  @protected
  Picture.library(this.data)
      : src = null,
        path = null;

  /// Define a new Picture with a reference to a
  /// local asset file. [path] is the file path of
  /// the target file.
  /// ```
  Picture.asset(this.path)
      : data = null,
        src = null;

  ///  Define a new Picture with a reference to a
  ///  URL link ie [src].
  /// ```
  Picture.network(this.src)
      : data = null,
        path = null;
}

extension LibraryGallery on Gallery {
  static void configure(
    Picture flash,
    Picture flashOff,
    Picture favorite,
    Picture favoriteFill,
    Picture check,
    Picture checkFill,
  ) {
    gallery.at(#flash)[Picture.library(Icons.flash_on)] = flash;
    gallery.at(#flashOff)[Picture.library(Icons.flash_off)] = flashOff;
    gallery.at(#favorite)[Picture.library(CupertinoIcons.suit_heart)] = favorite;
    gallery.at(#favoriteFill)[Picture.library(CupertinoIcons.suit_heart_fill)] = favoriteFill;
    gallery.at(#check)[Picture.library(CupertinoIcons.checkmark_square)] = flash;
    gallery.at(#checkFill)[Picture.library(CupertinoIcons.checkmark_square_fill)] = flash;
  }

  Picture get flash => gallery.at(#flash)[Picture.library(Icons.flash_on)];
  Picture get flashOff => gallery.at(#flash_off)[Picture.library(Icons.flash_off)];
  Picture get favorite => gallery.at(#favorite)[Picture.library(CupertinoIcons.suit_heart)];
  Picture get favoriteFill => gallery.at(#favoriteFill)[Picture.library(CupertinoIcons.suit_heart_fill)];
  Picture get check => gallery.at(#check)[Picture.library(CupertinoIcons.app)];
  Picture get checkFill => gallery.at(#checkFill)[Picture.library(CupertinoIcons.checkmark_square_fill)];
}
