part of nueli.core.bucket;

Palette get palette => Palette();

class Palette extends Bucket<Color> {
  static Symbol _themeChannel = #basic;
  static set theme(Symbol themeCode) => _themeChannel = themeCode;
  @override
  Symbol get iD => #palette;
}

class Swatch extends Color {
  Swatch.code(int basic, [Map<Symbol, int> others = const {}])
      : super({#basic: basic, ...others}[Palette._themeChannel] ?? basic);

  factory Swatch(Color basic, [Map<Symbol, Color> others = const {}]) {
    return Swatch.code(basic.value, others.map((key, value) => MapEntry(key, value.value)));
  }
}

extension NueliPalette on Palette {
  Color get nueliGrey => Swatch(const Color(0xff45494e), {#dark: const Color(0xffffffff)});
  Color get nueliOrange => Swatch(const Color(0xffe77d23), {#dark: const Color(0xfffdb880)});
}

extension MaterialPalette on Palette {
  static void configure({
    MaterialColor? material,
    MaterialAccentColor? materialAccent,
  }) {
    palette.at(#material)[Colors.blueGrey] = material;
    palette.at(#materialAccent)[Colors.orangeAccent] = materialAccent;
  }

  Color get material => palette.at(#material)[Colors.blue];
  Color get materialAccent => palette.at(#materialAccent)[Colors.tealAccent];
}

extension SurfacePalette on Palette {
  static void configure({
    Color? primary,
    Color? primaryLight,
    Color? primaryDark,
    Color? accent,
    Color? background,
  }) {
    palette.at(#primary)[palette.at(#material)[Colors.blueGrey]] = primary;
    palette.at(#primaryLight)[palette.at(#material)[Colors.blueGrey]] = primaryLight;
    palette.at(#primaryDark)[palette.at(#material)[Colors.blueGrey]] = primaryDark;
    palette.at(#accent)[palette.at(#materialAccent)[Colors.blueGrey]] = accent;
    palette.at(#background)[Colors.white] = background;
  }

  Color get primary => palette.at(#primary)[Colors.blueGrey];
  Color get primaryLight => palette.at(#primaryLight)[Colors.blueGrey];
  Color get primaryDark => palette.at(#primaryDark)[Colors.blueGrey];
  Color get accent => palette.at(#accent)[Colors.blueGrey];
  Color get background => palette.at(#background)[Colors.white];
}

extension ContentPalette on Palette {
  static void configure({
    Color? onPrimary,
    Color? onPrimaryLight,
    Color? onPrimaryDark,
    Color? onAccent,
    Color? onBackground,
  }) {
    palette.at(#onBackground)[Colors.blueGrey] = onBackground;
    palette.at(#onPrimary)[palette.at(#background)[Colors.blueGrey]] = onPrimary;
    palette.at(#onPrimaryLight)[palette.at(#onBackground)[Colors.blueGrey]] = onPrimaryLight;
    palette.at(#onPrimaryDark)[palette.at(#background)[Colors.blueGrey]] = onPrimaryDark;
    palette.at(#onAccent)[palette.at(#onBackground)[Colors.black54]] = onAccent;
  }

  Color get onBackground => palette.at(#onBackground)[Colors.blueGrey];
  Color get onPrimary => palette.at(#onPrimary)[Colors.blueGrey];
  Color get onPrimaryLight => palette.at(#onPrimaryLight)[Colors.blueGrey];
  Color get onPrimaryDark => palette.at(#onPrimaryDark)[Colors.blueGrey];
  Color get onAccent => palette.at(#onAccent)[Colors.blueGrey];
}

extension ColorPalette on Palette {
  static void configure({Color? shade, Color? tint}) {
    palette.at(#shade)[Colors.black54] = shade;
    palette.at(#tint)[Colors.white54] = tint;
  }

  Color get none => Colors.white.withAlpha(0);
  Color get white => Colors.white;
  Color get black => Colors.black;
  Color get grey => Colors.grey;
  Color get blueGrey => Colors.blueGrey;
  Color get shade => palette.at(#shade)[Colors.black54];
  Color get tint => palette.at(#tint)[Colors.white54];
}

extension StatusPalette on Palette {
  static void scheme({Color? neutral, Color? error, Color? success, Color? warning, Color? message}) {
    palette.at(#neutral)[palette.at(#accent)[Colors.blueGrey]] = neutral;
    palette.at(#error)[Colors.redAccent] = error;
    palette.at(#success)[Colors.green] = success;
    palette.at(#warning)[Colors.amber] = warning;
    palette.at(#message)[Colors.brown] = message;
  }

  Color get neutral => palette.at(#neutral)[Colors.blueGrey];
  Color get error => palette.at(#error)[Colors.redAccent];
  Color get success => palette.at(#success)[Colors.green];
  Color get warning => palette.at(#warning)[Colors.amber];
  Color get message => palette.at(#message)[Colors.brown];
}
