part of nueli.core.config;

mixin InkwellConfig<T extends InkwellConfig<T, IN, OUT>, IN extends InkwellConfigIN<IN, OUT>,
    OUT extends InkwellConfigOUT<OUT, IN>> on RadiiConfig<T, IN, OUT> {
  static int get defA => palette.tint.alpha;

  /// *@varies* ~ set splash color
  T splashColor(Color color, [int? alpha]) => resolve(color.withAlpha(alpha ?? defA), #splashColor);

  /// *@varies* ~ set hover color
  T hoverColor(Color color, [int? alpha]) => resolve(color.withAlpha(alpha ?? defA), #hoverColor);

  /// *@varies* ~ set focus color
  T focusColor(Color color, [int? alpha]) => resolve(color.withAlpha(alpha ?? defA), #focusColor);

  /// *@varies* ~ set overlay color
  T overlayColor(Color color, [int? alpha]) => resolve(color.withAlpha(alpha ?? defA), #overlayColor);

  /// *@varies* ~ set highlight color
  T highlightColor(Color color, [int? alpha]) => resolve(color.withAlpha(alpha ?? defA), #highlightColor);

  /// *@fixed* ~ set Inkwell type
  T inkType(InkType type) => resolve(type, #inkType);

  /// *@fixed* ~ set tap callback
  T onTap(Function() onTap) => resolve(onTap, #onTap);

  /// *@fixed* ~ set long press callback
  T onLongPress(Function() onLongPress) => resolve(onLongPress, #onLongPress);

  /// *@config reference* ~ copy inkwell
  T copyInkwell(InkwellConfig inkwell) {
    final iN = inkwell().iN as InkwellConfigIN;
    return update(
      (data) => data
        ..resolve(iN.splashColor, #splashColor)
        ..resolve(iN.hoverColor, #hoverColor)
        ..resolve(iN.focusColor, #focusColor)
        ..resolve(iN.overlayColor, #overlayColor)
        ..resolve(iN.highlightColor, #highlightColor)
        ..resolve(iN.inkType, #inkType)
        ..resolve(iN.onTap, #onTap)
        ..resolve(iN.onLongPress, #onLongPress),
    );
  }
}

mixin InkwellConfigIN<T extends InkwellConfigIN<T, OUT>, OUT extends InkwellConfigOUT<OUT, T>>
    on ConfigIN<T, OUT>, RadiiConfigIN<T, OUT> {
  Color? splashColor, hoverColor, focusColor, overlayColor, highlightColor;

  InkType? inkType;
  Function()? onTap, onLongPress;

  @override
  T operator *(double operand) {
    return (super * operand)
      ..splashColor = this().splashColor * operand
      ..hoverColor = this().hoverColor * operand
      ..focusColor = this().focusColor * operand
      ..overlayColor = this().overlayColor * operand
      ..highlightColor = this().highlightColor * operand;
  }

  @override
  T operator -(T operand) {
    return (super - operand)
      ..splashColor = this().splashColor - operand().splashColor
      ..hoverColor = this().hoverColor - operand().hoverColor
      ..focusColor = this().focusColor - operand().focusColor
      ..overlayColor = this().overlayColor - operand().overlayColor
      ..highlightColor = this().highlightColor - operand().highlightColor;
  }

  @override
  T operator +(T operand) {
    return (super + operand)
      ..splashColor = this().splashColor + operand().splashColor
      ..hoverColor = this().hoverColor + operand().hoverColor
      ..focusColor = this().focusColor + operand().focusColor
      ..overlayColor = this().overlayColor + operand().overlayColor
      ..highlightColor = this().highlightColor + operand().highlightColor;
  }

  @override
  T resolve(data, tag) {
    if (data is Color && tag == #splashColor) splashColor = data;
    if (data is Color && tag == #hoverColor) hoverColor = data;
    if (data is Color && tag == #focusColor) focusColor = data;
    if (data is Color && tag == #overlayColor) overlayColor = data;
    if (data is Color && tag == #highlightColor) highlightColor = data;

    if (data is InkType && tag == #inkType) inkType = data;
    if (data is Function() && tag == #onTap) onTap = data;
    if (data is Function() && tag == #onLongPress) onLongPress = data;
    return super.resolve(data, tag);
  }

  @override
  T copy(iN) {
    splashColor = iN.splashColor ?? splashColor;
    hoverColor = iN.hoverColor ?? hoverColor;
    focusColor = iN.focusColor ?? focusColor;
    overlayColor = iN.overlayColor ?? overlayColor;
    highlightColor = iN.highlightColor ?? highlightColor;

    inkType = iN.inkType ?? inkType;
    onTap = iN.onTap ?? onTap;
    onLongPress = iN.onLongPress ?? onLongPress;
    return super.copy(iN);
  }
}

mixin InkwellConfigOUT<T extends InkwellConfigOUT<T, IN>, IN extends InkwellConfigIN<IN, T>>
    on ConfigOUT<T, IN>, RadiiConfigOUT<T, IN> {
  static int get defA => palette.tint.alpha;

  Color get splashColor => iN.splashColor ?? palette.accent.withAlpha(defA);
  Color get hoverColor => iN.hoverColor ?? palette.accent.withAlpha(defA ~/ 2);
  Color get focusColor => iN.focusColor ?? palette.accent.withAlpha(defA ~/ 5);
  Color get overlayColor => iN.overlayColor ?? palette.accent.withAlpha(defA ~/ 2);
  Color get highlightColor => iN.highlightColor ?? palette.accent.withAlpha(defA ~/ 3);

  InkType get type => iN.inkType ?? InkType.well;
  Function() get onTap => iN.onTap ?? () {};
  Function() get onLongPress => iN.onLongPress ?? () {};

  GestureRecognizer? get gestureRecognizer {
    if (iN.onTap != null) return TapGestureRecognizer()..onTap = onTap;
    if (iN.onLongPress != null) return LongPressGestureRecognizer()..onLongPress = onLongPress;
    return null;
  }

  @override
  Widget inside(child) {
    if (gestureRecognizer == null) return super.inside(child);
    return type == InkType.response
        ? InkResponse(
            child: super.inside(child),
            containedInkWell: false,
            highlightColor: overlayColor,
            overlayColor: MaterialStateProperty.all<Color>(overlayColor),
            splashColor: splashColor,
            hoverColor: hoverColor,
            focusColor: focusColor,
            onTap: onTap,
            onLongPress: onLongPress,
          )
        : Material(
            color: palette.none,
            child: InkWell(
              child: super.inside(child),
              borderRadius: borderRadius,
              splashColor: splashColor,
              hoverColor: hoverColor,
              focusColor: focusColor,
              overlayColor: MaterialStateProperty.all<Color>(overlayColor),
              highlightColor: highlightColor,
              onTap: onTap,
              onLongPress: onLongPress,
            ),
          );
  }
}

enum InkType { response, well }
