part of nueli.core.config;

/// Photo ~ [Image] and [Icon] configuration class.
mixin PhotoConfig<T extends PhotoConfig<T, IN, OUT>, IN extends PhotoConfigIN<IN, OUT>, OUT extends PhotoConfigOUT<OUT, IN>>
    on LayoutConfig<T, IN, OUT>, InkwellConfig<T, IN, OUT> {
  /// *@varies* ~ set color filter
  T filter(Color color) => resolve(color, #filter).filterBlend(BlendMode.srcIn);

  /// *@fixed* ~ set blend mode
  T filterBlend(BlendMode? blendMode) => resolve(blendMode, #filterBlend);

  /// *@fixed* ~ set picture data
  T picture(Picture? picture) => resolve(picture, #picture);

  /// *@fixed* ~ set box fit
  T pictureFit(BoxFit? boxFit) => resolve(boxFit, #pictureFit);
}

mixin PhotoConfigIN<T extends PhotoConfigIN<T, OUT>, OUT extends PhotoConfigOUT<OUT, T>>
    on LayoutConfigIN<T, OUT>, InkwellConfigIN<T, OUT> {
  Color? filter;

  BlendMode? filterBlend;
  Picture? picture;
  BoxFit? pictureFit;

  @override
  T operator *(double operand) {
    return (super * operand)..filter = this().filter != null ? this().filter! * operand : null;
  }

  @override
  T operator -(T operand) {
    return (super - operand)..filter = this().filter != null ? this().filter! - (operand().filter ?? this().filter!) : null;
  }

  @override
  T operator +(T operand) {
    return (super + operand)..filter = this().filter != null ? this().filter! + (operand().filter ?? this().filter!) : null;
  }

  @override
  T resolve(data, tag) {
    if (data is Color && tag == #filter) filter = data;

    if (data is BlendMode && tag == #filterBlend) filterBlend = data;
    if (data is Picture && tag == #picture) picture = data;
    if (data is BoxFit && tag == #pictureFit) pictureFit = data;

    return super.resolve(data, tag);
  }

  @override
  T copy(iN) {
    filter = iN.filter ?? filter;

    picture = iN.picture ?? picture;
    filterBlend = iN.filterBlend ?? filterBlend;
    pictureFit = iN.pictureFit ?? pictureFit;
    return super.copy(iN);
  }
}

mixin PhotoConfigOUT<T extends PhotoConfigOUT<T, IN>, IN extends PhotoConfigIN<IN, T>>
    on LayoutConfigOUT<T, IN>, InkwellConfigOUT<T, IN> {
  Color? get filter => iN.filter;

  BlendMode get filterBlend => iN.filterBlend ?? BlendMode.srcIn;

  Picture get picture => iN.picture ?? Picture.error;

  BoxFit get pictureFit => iN.pictureFit ?? BoxFit.contain;

  @override
  Widget combine(children) {
    Widget result = const SizedBox();

    /// library
    if (picture.data != null) {
      result = Icon(
        picture.data,
        color: filter,
        size: shortestSide,
      );
    }

    /// asset
    if (picture.path != null) {
      result = Image.asset(
        picture.path!,
        fit: pictureFit,
        alignment: alignment ?? Alignment.center,
        color: filter,
        colorBlendMode: filterBlend,
        width: width,
        height: height,
      );
    }

    /// network
    if (picture.src != null) {
      Widget loader(ImageChunkEvent chunk) {
        final eTB = chunk.expectedTotalBytes;
        final cBL = chunk.cumulativeBytesLoaded;
        final loadValue = eTB == null ? cBL / eTB! : null;
        return Center(
          child: Container(
            height: 36,
            width: 36,
            padding: const EdgeInsets.all(8.0),
            child: CircularProgressIndicator(
              strokeWidth: 2,
              value: loadValue,
              color: palette.accent,
              backgroundColor: palette.background,
            ),
          ),
        );
      }

      Widget error([StackTrace? trace]) {
        return Center(
          child: Padding(
            padding: const EdgeInsets.all(8.0),
            child: ErrorWidget((trace ?? 'An error occurred').toString()),
          ),
        );
      }

      result = Image.network(
        picture.src!,
        color: filter,
        fit: pictureFit,
        alignment: alignment ?? Alignment.center,
        colorBlendMode: filterBlend,
        height: height,
        width: width,
        errorBuilder: (c, o, s) => error(s),
        loadingBuilder: (c, child, chunk) => chunk == null ? child : loader(chunk),
      );
    }
    return super.combine([result, ...children]);
  }
}
