part of nueli.core.config;

mixin RadiiConfig<T extends RadiiConfig<T, IN, OUT>, IN extends RadiiConfigIN<IN, OUT>, OUT extends RadiiConfigOUT<OUT, IN>>
    on Config<T, IN, OUT> {
  T roundTopLeftX(double value) => resolve(value, #roundTopLeftX);

  T roundTopLeftY(double value) => resolve(value, #roundTopLeftY);

  T roundTopRightX(double value) => resolve(value, #roundTopRightX);

  T roundTopRightY(double value) => resolve(value, #roundTopRightY);

  T roundBottomRightX(double value) => resolve(value, #roundBottomRightX);

  T roundBottomRightY(double value) => resolve(value, #roundBottomRightY);

  T roundBottomLeftX(double value) => resolve(value, #roundBottomLeftX);

  T roundBottomLeftY(double value) => resolve(value, #roundBottomLeftY);

  T roundTopLeft(double x, [double? y]) => roundTopLeftX(x).roundTopLeftY(y ?? x);

  T roundTopRight(double x, [double? y]) => roundTopRightX(x).roundTopRightY(y ?? x);

  T roundBottomRight(double x, [double? y]) => roundBottomRightX(x).roundBottomRightY(y ?? x);

  T roundBottomLeft(double x, [double? y]) => roundBottomLeftX(x).roundBottomLeftY(y ?? x);

  T roundLeft(double value) => roundTopLeftY(value).roundBottomLeftY(value);

  T roundRight(double value) => roundTopRightY(value).roundBottomRightY(value);

  T roundTop(double value) => roundTopLeftX(value).roundTopRightX(value);

  T roundBottom(double value) => roundBottomLeftX(value).roundBottomRightX(value);

  /// *@fixed* ~ set clip behaviour
  T clip(Clip? clip) => resolve(clip, #clip);

  /// *@config reference* ~ copy radii
  T copyRadii(RadiiConfig radii) {
    final iN = radii().iN as RadiiConfigIN;
    return update(
      (data) => data
        ..resolve(iN.roundTopLeftX, #roundTopLeftX)
        ..resolve(iN.roundTopLeftY, #roundTopLeftY)
        ..resolve(iN.roundTopRightX, #roundTopRightX)
        ..resolve(iN.roundTopRightY, #roundTopRightY)
        ..resolve(iN.roundBottomRightX, #roundBottomRightX)
        ..resolve(iN.roundBottomRightY, #roundBottomRightY)
        ..resolve(iN.roundBottomLeftX, #roundBottomLeftX)
        ..resolve(iN.roundBottomLeftY, #roundBottomLeftY),
    );
  }
}

mixin RadiiConfigIN<T extends RadiiConfigIN<T, OUT>, OUT extends RadiiConfigOUT<OUT, T>> on ConfigIN<T, OUT> {
  double? roundTopLeftX;
  double? roundTopLeftY;
  double? roundTopRightX;
  double? roundTopRightY;
  double? roundBottomRightX;
  double? roundBottomRightY;
  double? roundBottomLeftX;
  double? roundBottomLeftY;
  Clip? clip;

  @override
  T operator *(double operand) {
    return (super * operand)
      ..roundTopLeftX = this().roundTopLeftX * operand
      ..roundTopLeftY = this().roundTopLeftY * operand
      ..roundTopRightX = this().roundTopRightX * operand
      ..roundTopRightY = this().roundTopRightY * operand
      ..roundBottomRightX = this().roundBottomRightX * operand
      ..roundBottomRightY = this().roundBottomRightY * operand
      ..roundBottomLeftX = this().roundBottomLeftX * operand
      ..roundBottomLeftY = this().roundBottomLeftY * operand;
  }

  @override
  T operator -(T operand) {
    return (super - operand)
      ..roundTopLeftX = this().roundTopLeftX - operand().roundTopLeftX
      ..roundTopLeftY = this().roundTopLeftY - operand().roundTopLeftY
      ..roundTopRightX = this().roundTopRightX - operand().roundTopRightX
      ..roundTopRightY = this().roundTopRightY - operand().roundTopRightY
      ..roundBottomRightX = this().roundBottomRightX - operand().roundBottomRightX
      ..roundBottomRightY = this().roundBottomRightY - operand().roundBottomRightY
      ..roundBottomLeftX = this().roundBottomLeftX - operand().roundBottomLeftX
      ..roundBottomLeftY = this().roundBottomLeftY - operand().roundBottomLeftY;
  }

  @override
  T operator +(T operand) {
    return (super + operand)
      ..roundTopLeftX = this().roundTopLeftX + operand().roundTopLeftX
      ..roundTopLeftY = this().roundTopLeftY + operand().roundTopLeftY
      ..roundTopRightX = this().roundTopRightX + operand().roundTopRightX
      ..roundTopRightY = this().roundTopRightY + operand().roundTopRightY
      ..roundBottomRightX = this().roundBottomRightX + operand().roundBottomRightX
      ..roundBottomRightY = this().roundBottomRightY + operand().roundBottomRightY
      ..roundBottomLeftX = this().roundBottomLeftX + operand().roundBottomLeftX
      ..roundBottomLeftY = this().roundBottomLeftY + operand().roundBottomLeftY;
  }

  @override
  T resolve(data, Symbol tag) {
    if (data is double && tag == #roundTopLeftX) roundTopLeftX = data;
    if (data is double && tag == #roundTopLeftY) roundTopLeftY = data;
    if (data is double && tag == #roundTopRightX) roundTopRightX = data;
    if (data is double && tag == #roundTopRightY) roundTopRightY = data;
    if (data is double && tag == #roundBottomRightX) roundBottomRightX = data;
    if (data is double && tag == #roundBottomRightY) roundBottomRightY = data;
    if (data is double && tag == #roundBottomLeftX) roundBottomLeftX = data;
    if (data is double && tag == #roundBottomLeftY) roundBottomLeftY = data;
    if (data is Clip && tag == #clip) clip = data;
    return super.resolve(data, tag);
  }

  @override
  T copy(iN) {
    roundTopLeftX = iN.roundTopLeftX ?? roundTopLeftX;
    roundTopLeftY = iN.roundTopLeftY ?? roundTopLeftY;
    roundTopRightX = iN.roundTopRightX ?? roundTopRightX;
    roundTopRightY = iN.roundTopRightY ?? roundTopRightY;
    roundBottomRightX = iN.roundBottomRightX ?? roundBottomRightX;
    roundBottomRightY = iN.roundBottomRightY ?? roundBottomRightY;
    roundBottomLeftX = iN.roundBottomLeftX ?? roundBottomLeftX;
    roundBottomLeftY = iN.roundBottomLeftY ?? roundBottomLeftY;
    clip = iN.clip ?? clip;
    return super.copy(iN);
  }
}

mixin RadiiConfigOUT<T extends RadiiConfigOUT<T, IN>, IN extends RadiiConfigIN<IN, T>> on ConfigOUT<T, IN> {
  double get roundTopLeftX => iN.roundTopLeftX ?? 0;

  double get roundTopLeftY => iN.roundTopLeftX ?? 0;

  double get roundTopRightX => iN.roundTopRightX ?? 0;

  double get roundTopRightY => iN.roundTopRightX ?? 0;

  double get roundBottomLeftX => iN.roundBottomLeftX ?? 0;

  double get roundBottomLeftY => iN.roundBottomLeftX ?? 0;

  double get roundBottomRightX => iN.roundBottomRightX ?? 0;

  double get roundBottomRightY => iN.roundBottomRightX ?? 0;

  Clip get clip => iN.clip ?? Clip.antiAlias;

  BorderRadius get borderRadius {
    return BorderRadius.only(
      topLeft: Radius.elliptical(roundTopLeftX, roundTopLeftY),
      topRight: Radius.elliptical(roundTopRightX, roundTopRightY),
      bottomLeft: Radius.elliptical(roundBottomLeftX, roundBottomLeftY),
      bottomRight: Radius.elliptical(roundBottomRightX, roundBottomRightY),
    );
  }

  @override
  Widget inside(child) {
    return Container(
      child: super.inside(child),
      decoration: BoxDecoration(borderRadius: borderRadius),
      clipBehavior: clip,
    );
  }
}
