part of nueli.core.config;

/// Txt ~ [Text] and [TextStyle] configuration class.
mixin TxtConfig<T extends TxtConfig<T, IN, OUT>, IN extends TxtConfigIN<IN, OUT>, OUT extends TxtConfigOUT<OUT, IN>>
    on LayoutConfig<T, IN, OUT>, InkwellConfig<T, IN, OUT> {
  /// *@fixed* ~ set text
  T text(String text) => resolve(text, #text);

  /// *@varies* ~ set text size
  T textSize(double size) => resolve(size, #textSize);

  /// *@varies* ~ set text color
  T textColor(Color color) => resolve(color, #textColor);

  /// *@varies* ~ set text background
  T textBackground(Color color) => resolve(color, #textBackground);

  /// *@fixed* ~ add text decoration
  T textDecoration(TextDecoration decoration) {
    return resolve(TextDecoration.combine([this().textDecoration ?? TextDecoration.none, decoration]), #textDecoration);
  }

  /// *@fixed* ~ add text decoration color
  T textDecorationColor(Color color) {
    return resolve(color, #textDecorationColor);
  }

  /// *@fixed* ~ add text decoration color
  T textDecorationThickness(double thickness) {
    return resolve(thickness, #textDecorationThickness);
  }

  /// *@fixed* ~ set text baseline
  T textBaseline(TextBaseline baseline) => resolve(baseline, #textBaseline);

  /// *@fixed* ~ set text overflow
  T textOverflow(TextOverflow? overflow) => resolve(overflow, #textOverflow);

  /// *@fixed* ~ set text align
  T textAlign(TextAlign align) => resolve(align, #textAlign);

  /// *@fixed* ~ set text line count
  T textLineCount(int count) => resolve(count, #textLineCount);

  /// *@fixed* ~ set font weight
  T fontWeight(FontWeight? weight) => resolve(weight, #fontWeight);

  /// *@fixed* ~ set font style
  T fontStyle(FontStyle? style) => resolve(style, #fontStyle);

  /// *@fixed* ~ set text span children
  T inline(List<Widget> children) => resolve(children, #inline);

  /// *@shortcut* ~ add underline to decorations
  T get underline => textDecoration(TextDecoration.underline);

  /// *@shortcut* ~ add overline to decorations
  T get overline => textDecoration(TextDecoration.overline);

  /// *@shortcut* ~ add lineThrough to decorations
  T get lineThrough => textDecoration(TextDecoration.lineThrough);

  /// *@shortcut* ~ set font weight to bold
  T get bold => fontWeight(FontWeight.bold);

  /// *@shortcut* ~ set font style to italics
  T get italics => fontStyle(FontStyle.italic);
}

mixin TxtConfigIN<T extends TxtConfigIN<T, OUT>, OUT extends TxtConfigOUT<OUT, T>>
    on LayoutConfigIN<T, OUT>, InkwellConfigIN<T, OUT> {
  String? text;
  double? textSize;
  Color? textColor, textBackground;
  TextBaseline? textBaseline;
  TextOverflow? textOverflow;
  TextAlign? textAlign;
  int? textLineCount;
  TextDecoration? textDecoration;
  Color? textDecorationColor;
  double? textDecorationThickness;
  FontWeight? fontWeight;
  FontStyle? fontStyle;
  List<Widget>? inline;

  @override
  T operator *(double operand) {
    return (super * operand)
      ..textSize = this().textSize != null ? this().textSize! * operand : null
      ..textColor = this().textColor != null ? this().textColor! * operand : null
      ..textBackground = this().textBackground != null ? this().textBackground! * operand : null
      ..textDecorationColor = this().textDecorationColor != null ? this().textDecorationColor! * operand : null
      ..textDecorationThickness = this().width != null ? this().textDecorationThickness! * operand : null;
  }

  @override
  T operator -(T operand) {
    return (super - operand)
      ..textSize = this().textSize != null ? this().textSize! - (operand().textSize ?? this().textSize!) : null
      ..textColor = this().textColor != null ? this().textColor! - (operand().textColor ?? this().textColor!) : null
      ..textBackground = this().textBackground != null
          ? this().textBackground! - (operand().textBackground ?? this().textBackground!)
          : null
      ..textDecorationColor = this().textDecorationColor != null
          ? this().textDecorationColor! - (operand().textDecorationColor ?? this().textDecorationColor!)
          : null
      ..textDecorationThickness = this().textDecorationThickness != null
          ? this().textDecorationThickness! - (operand().textDecorationThickness ?? this().textDecorationThickness!)
          : null;
  }

  @override
  T operator +(T operand) {
    return (super + operand)
      ..textSize = this().textSize != null ? this().textSize! + (operand().textSize ?? this().textSize!) : null
      ..textColor = this().textColor != null ? this().textColor! + (operand().textColor ?? this().textColor!) : null
      ..textBackground = this().textBackground != null
          ? this().textBackground! + (operand().textBackground ?? this().textBackground!)
          : null
      ..textDecorationColor = this().textDecorationColor != null
          ? this().textDecorationColor! + (operand().textDecorationColor ?? this().textDecorationColor!)
          : null
      ..textDecorationThickness = this().textDecorationThickness != null
          ? this().textDecorationThickness! + (operand().textDecorationThickness ?? this().textDecorationThickness!)
          : null;
  }

  @override
  T resolve(data, tag) {
    if (data is String && tag == #text) text = data;
    if (data is double && tag == #textSize) textSize = data;
    if (data is Color && tag == #textColor) textColor = data;
    if (data is Color && tag == #textBackground) textBackground = data;
    if (data is TextAlign && tag == #textAlign) textAlign = data;
    if (data is int && tag == #textLineCount) textLineCount = data;
    if (data is TextDecoration && tag == #textDecoration) textDecoration = data;
    if (data is Color && tag == #textDecorationColor) textDecorationColor = data;
    if (data is double && tag == #textDecorationThickness) textDecorationThickness = data;
    if (data is TextBaseline && tag == #textBaseline) textBaseline = data;
    if (data is TextOverflow && tag == #textOverflow) textOverflow = data;
    if (data is FontWeight && tag == #fontWeight) fontWeight = data;
    if (data is FontStyle && tag == #fontStyle) fontStyle = data;
    if (data is List<Widget> && tag == #inline) inline = data;
    return super.resolve(data, tag);
  }

  @override
  T copy(iN) {
    text = iN.text ?? text;
    textSize = iN.textSize ?? textSize;
    textColor = iN.textColor ?? textColor;
    textBackground = iN.textBackground ?? textBackground;
    textAlign = iN.textAlign ?? textAlign;
    textLineCount = iN.textLineCount ?? textLineCount;
    textDecoration = iN.textDecoration ?? textDecoration;
    textDecorationColor = iN.textDecorationColor ?? textDecorationColor;
    textDecorationThickness = iN.textDecorationThickness ?? textDecorationThickness;
    textBaseline = iN.textBaseline ?? textBaseline;
    textOverflow = iN.textOverflow ?? textOverflow;
    fontWeight = iN.fontWeight ?? fontWeight;
    fontStyle = iN.fontStyle ?? fontStyle;
    inline = iN.inline ?? inline;
    return super.copy(iN);
  }
}

mixin TxtConfigOUT<T extends TxtConfigOUT<T, IN>, IN extends TxtConfigIN<IN, T>>
    on LayoutConfigOUT<T, IN>, InkwellConfigOUT<T, IN> {
  static List<TextStyle> rx = [];

  String get text => iN.text ?? '';

  double? get textSize => iN.textSize ?? 12;

  Color? get textColor => iN.textColor;

  Color? get textBackground => iN.textBackground;

  TextAlign get textAlign => iN.textAlign ?? TextAlign.justify;

  int? get textLineCount => iN.textLineCount;

  TextDecoration? get textDecoration => iN.textDecoration;

  Color? get textDecorationColor => iN.textDecorationColor;

  double? get textDecorationThickness => iN.textDecorationThickness;

  TextBaseline? get textBaseline => iN.textBaseline;

  TextOverflow? get textOverflow => iN.textOverflow ?? TextOverflow.ellipsis;

  FontWeight get fontWeight => iN.fontWeight ?? FontWeight.normal;

  FontStyle? get fontStyle => iN.fontStyle ?? FontStyle.normal;

  List<Widget> get inline => iN.inline ?? [];

  TextStyle get textStyle {
    return TextStyle(
      color: textColor,
      fontSize: textSize,
      fontStyle: fontStyle,
      fontWeight: fontWeight,
      backgroundColor: textBackground,
      decoration: textDecoration,
      decorationColor: textDecorationColor,
      decorationThickness: textDecorationThickness,
      textBaseline: textBaseline,
      overflow: textOverflow,
    );
  }

  InlineSpan get textSpan {
    List<InlineSpan> spans = [];

    if (inline.isNotEmpty) rx.add(textStyle);
    for (int i = 0; i < inline.length; i++) {
      final txt = inline[i];
      spans.add(txt is TxtConfig ? txt().textSpan : WidgetSpan(child: txt, style: textStyle));
      if (i == inline.length - 1) rx.removeLast();
    }

    return TextSpan(
      text: text,
      style: rx.reversed.fold(textStyle, (p, e) => p!.overlayOn(e)),
      recognizer: gestureRecognizer,
      children: spans.isEmpty ? null : spans,
    );
  }

  @override
  Widget combine(children) {
    Widget child = Text(
      text,
      textAlign: textAlign,
      maxLines: textLineCount,
      overflow: textOverflow,
      style: textStyle,
    );
    if (inline.isNotEmpty) {
      child = Text.rich(
        textSpan,
        textAlign: textAlign,
        maxLines: textLineCount,
        overflow: textOverflow,
        style: textStyle,
      );
    }
    return super.combine([child, ...children]);
  }
}

extension _TextStyle on TextStyle {
  TextStyle overlayOn(TextStyle? ts) {
    if (ts == null) return this;
    return TextStyle(
      color: color ?? ts.color,
      fontSize: fontSize ?? ts.fontSize,
      fontStyle: fontStyle ?? ts.fontStyle,
      fontWeight: fontWeight ?? ts.fontWeight,
      backgroundColor: backgroundColor ?? ts.backgroundColor,
      decoration: decoration ?? ts.decoration,
      decorationColor: decorationColor ?? ts.decorationColor,
      decorationThickness: decorationThickness ?? ts.decorationThickness,
      textBaseline: textBaseline ?? ts.textBaseline,
      overflow: overflow ?? ts.overflow,
    );
  }
}
