part of nueli.core.config;

mixin X3DConfig<T extends X3DConfig<T, IN, OUT>, IN extends X3DConfigIN<IN, OUT>, OUT extends X3DConfigOUT<OUT, IN>>
    on Config<T, IN, OUT> {
  T zoom(double zoom) => resolve(TransformData(zoom, (t, d) => t..setRotationZ(d)), #zoom);

  T rotateX(double x) => resolve(TransformData(x, (t, d) => t..setRotationX(d)), #rotateX);

  T rotateY(double y) => resolve(TransformData(y, (t, d) => t..setRotationY(d)), #rotateY);

  T translateX(double x) => resolve(
      TransformData(x, (t, d) => t..setTranslationRaw(d, this().translateY.value, this().translateZ.value)), #translateX);

  T translateY(double y) => resolve(
      TransformData(y, (t, d) => t..setTranslationRaw(this().translateY.value, d, this().translateZ.value)), #translateY);

  T translateZ(double z) => resolve(
      TransformData(z, (t, d) => t..setTranslationRaw(this().translateY.value, this().translateZ.value, d)), #translateZ);

  T transformAlignment(Alignment alignment) => resolve(alignment, #transformAlignment);

  T baseMatrix4(Matrix4? base) => resolve(base, #base);

  T rotate(double x, [double? y]) => rotateX(x).rotateY(y ?? x);

  T translate(double x, [double? y, double? z]) => translateX(x).translateY(y ?? x).translateZ(z ?? y ?? x);
}

mixin X3DConfigIN<T extends X3DConfigIN<T, OUT>, OUT extends X3DConfigOUT<OUT, T>> on ConfigIN<T, OUT> {
  TransformData? zoom;
  TransformData? rotateX;
  TransformData? rotateY;
  TransformData? translateX;
  TransformData? translateY;
  TransformData? translateZ;
  Alignment? transformAlignment;

  Matrix4? baseMatrix4;

  @override
  T operator *(double operand) {
    return (super * operand)
      ..zoom = this().zoom * operand
      ..rotateX = this().rotateX * operand
      ..rotateY = this().rotateY * operand
      ..translateX = this().translateX * operand
      ..translateY = this().translateY * operand
      ..translateZ = this().translateZ * operand
      ..transformAlignment = this().transformAlignment * operand;
  }

  @override
  T operator -(T operand) {
    return (super - operand)
      ..zoom = this().zoom - operand().zoom
      ..rotateX = this().rotateX - operand().rotateX
      ..rotateY = this().rotateY - operand().rotateY
      ..translateX = this().translateX - operand().translateX
      ..translateY = this().translateY - operand().translateY
      ..translateZ = this().translateZ - operand().translateZ
      ..transformAlignment = this().transformAlignment - this().transformAlignment;
  }

  @override
  T operator +(T operand) {
    return (super + operand)
      ..zoom = this().zoom + operand().zoom
      ..rotateX = this().rotateX + operand().rotateX
      ..rotateY = this().rotateY + operand().rotateY
      ..translateX = this().translateX + operand().translateX
      ..translateY = this().translateY + operand().translateY
      ..translateZ = this().translateZ + operand().translateZ
      ..transformAlignment = this().transformAlignment + operand().transformAlignment;
  }

  @override
  T resolve(data, tag) {
    if (data is TransformData && tag == #zoom) zoom = data;
    if (data is TransformData && tag == #rotateX) rotateX = data;
    if (data is TransformData && tag == #rotateY) rotateY = data;
    if (data is TransformData && tag == #translateX) translateX = data;
    if (data is TransformData && tag == #translateY) translateY = data;
    if (data is TransformData && tag == #translateZ) translateZ = data;
    if (data is Alignment && tag == #alignment) transformAlignment = data;
    if (data is Matrix4 && tag == #base) baseMatrix4 = data;
    return super.resolve(data, tag);
  }

  @override
  T copy(iN) {
    zoom = iN.zoom ?? zoom;
    rotateX = iN.rotateX ?? rotateX;
    rotateY = iN.rotateY ?? rotateY;
    translateX = iN.translateX ?? translateX;
    translateY = iN.translateY ?? translateY;
    translateZ = iN.translateZ ?? translateZ;
    transformAlignment = iN.transformAlignment ?? transformAlignment;
    baseMatrix4 = iN.baseMatrix4 ?? baseMatrix4;
    return super.copy(iN);
  }
}

mixin X3DConfigOUT<T extends X3DConfigOUT<T, IN>, IN extends X3DConfigIN<IN, T>> on ConfigOUT<T, IN> {
  TransformData get zoom => iN.zoom ?? TransformData(0, (t, d) => t);

  TransformData get rotateX => iN.rotateX ?? TransformData(0, (t, d) => t);

  TransformData get rotateY => iN.rotateY ?? TransformData(0, (t, d) => t);

  TransformData get translateX => iN.translateX ?? TransformData(0, (t, d) => t);

  TransformData get translateY => iN.translateY ?? TransformData(0, (t, d) => t);

  TransformData get translateZ => iN.translateZ ?? TransformData(0, (t, d) => t);

  Alignment get transformAlignment => iN.transformAlignment ?? Alignment.center;

  Matrix4 get base => iN.baseMatrix4 ?? (Matrix4.identity()..setEntry(3, 2, 0.0011));

  Matrix4 get toMatrix4 {
    Matrix4 mat = base;
    final transforms = [zoom, rotateX, rotateY, translateX, translateY, translateZ];
    for (int i = 0; i < transforms.length; mat = transforms[i](mat), i++) {}
    return mat;
  }

  @override
  Widget inside(child) {
    return Transform(
      child: super.inside(child),
      alignment: transformAlignment,
      transform: toMatrix4,
    );
  }
}

class TransformData with Subtract<TransformData>, Add<TransformData> {
  double value;
  Matrix4 Function(Matrix4, double) transform;

  TransformData(this.value, this.transform);

  Matrix4 call(Matrix4 matrix) => transform(matrix, value);

  @override
  TransformData operator -(TransformData operand) {
    return TransformData(value - operand.value, (mat, d) => transform(mat, d));
  }

  @override
  TransformData operator +(TransformData operand) {
    return TransformData(value + operand.value, (mat, d) => transform(mat, d));
  }

  TransformData operator *(double factor) {
    return TransformData(value * factor, (mat, d) => transform(mat, d));
  }
}
