import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:nueli_core/nueli_core.dart';

import '../config.dart';

Layout get $layout => Layout();

Decor get $decor => Decor();

Inkwell get $inkwell => Inkwell();

Photo get $photo => Photo();

Radii get $radii => Radii();

Txt get $txt => Txt();

Umbra get $umbra => Umbra();

X3D get $3D => X3D();

/// Layout ~ [LayoutConfig] final class.
class Layout extends Config<Layout, LayoutIN, LayoutOUT> with ComplexConfig, LayoutConfig {
  Layout({Key? key}) : super(key: key, fallback: LayoutIN());

  @override
  Layout createNew() => Layout();
}

class LayoutIN extends ConfigIN<LayoutIN, LayoutOUT> with ComplexConfigIN, LayoutConfigIN {
  @override
  LayoutIN createNew() => LayoutIN();

  @override
  LayoutOUT call() => LayoutOUT(this);
}

class LayoutOUT extends ConfigOUT<LayoutOUT, LayoutIN> with ComplexConfigOUT, LayoutConfigOUT {
  LayoutOUT(LayoutIN iN) : super(iN);
}

/// Decor ~ [DecorConfig] final class.
class Decor extends Config<Decor, DecorIN, DecorOUT>
    with ComplexConfig, RadiiConfig, UmbraConfig, InkwellConfig, LayoutConfig, DecorConfig {
  Decor({Key? key}) : super(key: key, fallback: DecorIN());

  @override
  Decor createNew() => Decor();
}

class DecorIN extends ConfigIN<DecorIN, DecorOUT>
    with ComplexConfigIN, RadiiConfigIN, UmbraConfigIN, InkwellConfigIN, LayoutConfigIN, DecorConfigIN {
  @override
  DecorIN createNew() => DecorIN();

  @override
  DecorOUT call() => DecorOUT(this);
}

class DecorOUT extends ConfigOUT<DecorOUT, DecorIN>
    with ComplexConfigOUT, RadiiConfigOUT, UmbraConfigOUT, InkwellConfigOUT, LayoutConfigOUT, DecorConfigOUT {
  DecorOUT(DecorIN iN) : super(iN);
}

/// Inkwell ~ [InkwellConfig] final class.
class Inkwell extends Config<Inkwell, InkwellIN, InkwellOUT> with RadiiConfig, InkwellConfig {
  Inkwell({Key? key}) : super(key: key, fallback: InkwellIN());

  factory Inkwell.ripple(Color color, [int alpha = 100]) {
    return Inkwell()
      ..splashColor(color, alpha).highlightColor(color, alpha ~/ 5)
      ..hoverColor(color, alpha ~/ 5).overlayColor(color, alpha ~/ 3)
      ..focusColor(color, alpha ~/ 5);
  }

  factory Inkwell.hover(Color color, [int alpha = 100]) {
    return Inkwell()
      ..splashColor(color, alpha ~/ 3).highlightColor(color, alpha ~/ 3.5)
      ..hoverColor(color, alpha).overlayColor(color, alpha ~/ 5)
      ..focusColor(color, alpha ~/ 3);
  }

  static Inkwell get clear {
    return Inkwell()
      ..splashColor(palette.white, 0).highlightColor(palette.white, 0)
      ..hoverColor(palette.white, 0).overlayColor(palette.white, 0)
      ..focusColor(palette.white, 0);
  }

  @override
  Inkwell createNew() => Inkwell();
}

class InkwellIN extends ConfigIN<InkwellIN, InkwellOUT> with RadiiConfigIN, InkwellConfigIN {
  @override
  InkwellIN createNew() => InkwellIN();

  @override
  InkwellOUT call() => InkwellOUT(this);
}

class InkwellOUT extends ConfigOUT<InkwellOUT, InkwellIN> with RadiiConfigOUT, InkwellConfigOUT {
  InkwellOUT(InkwellIN iN) : super(iN);
}

/// Photo ~ [PhotoConfig] final class
class Photo extends Config<Photo, PhotoIN, PhotoOUT>
    with ComplexConfig, RadiiConfig, InkwellConfig, LayoutConfig, PhotoConfig {
  Photo({Key? key}) : super(key: key, fallback: PhotoIN());

  @override
  Photo createNew() => Photo();
}

class PhotoIN extends ConfigIN<PhotoIN, PhotoOUT>
    with ComplexConfigIN, RadiiConfigIN, InkwellConfigIN, LayoutConfigIN, PhotoConfigIN {
  @override
  PhotoIN createNew() => PhotoIN();

  @override
  PhotoOUT call() => PhotoOUT(this);
}

class PhotoOUT extends ConfigOUT<PhotoOUT, PhotoIN>
    with ComplexConfigOUT, RadiiConfigOUT, InkwellConfigOUT, LayoutConfigOUT, PhotoConfigOUT {
  PhotoOUT(PhotoIN iN) : super(iN);
}

/// Radii ~ [RadiiConfig] final class
class Radii extends Config<Radii, RadiiIN, RadiiOUT> with RadiiConfig {
  Radii({Key? key}) : super(key: key, fallback: RadiiIN());

  factory Radii.all(double value) {
    return Radii().roundLeft(value).roundRight(value).roundTop(value).roundBottom(value);
  }

  factory Radii.sides({
    double left = 0,
    double right = 0,
    double top = 0,
    double bottom = 0,
  }) {
    return Radii()
      ..roundLeft(left).roundTop(top)
      ..roundTop(top).roundBottom(bottom);
  }

  factory Radii.corners({
    double topLeft = 0,
    double topRight = 0,
    double bottomRight = 0,
    double bottomLeft = 0,
  }) {
    return Radii()
      ..roundTopLeft(topLeft)
      ..roundTopRight(topRight)
      ..roundBottomRight(bottomRight)
      ..roundBottomLeft(bottomLeft);
  }

  @override
  Radii createNew() => Radii();
}

class RadiiIN extends ConfigIN<RadiiIN, RadiiOUT> with RadiiConfigIN {
  @override
  RadiiIN createNew() => RadiiIN();

  @override
  RadiiOUT call() => RadiiOUT(this);
}

class RadiiOUT extends ConfigOUT<RadiiOUT, RadiiIN> with RadiiConfigOUT {
  RadiiOUT(RadiiIN iN) : super(iN);
}

/// Txt ~ [TxtConfig] final class
class Txt extends Config<Txt, TxtIN, TxtOUT> with ComplexConfig, RadiiConfig, InkwellConfig, LayoutConfig, TxtConfig {
  Txt({Key? key}) : super(key: key, fallback: TxtIN());

  @override
  Txt createNew() => Txt();
}

class TxtIN extends ConfigIN<TxtIN, TxtOUT>
    with ComplexConfigIN, RadiiConfigIN, InkwellConfigIN, LayoutConfigIN, TxtConfigIN {
  @override
  TxtIN createNew() => TxtIN();

  @override
  TxtOUT call() => TxtOUT(this);
}

class TxtOUT extends ConfigOUT<TxtOUT, TxtIN>
    with ComplexConfigOUT, RadiiConfigOUT, InkwellConfigOUT, LayoutConfigOUT, TxtConfigOUT {
  TxtOUT(TxtIN iN) : super(iN);
}

/// Umbra ~ [UmbraConfig] final class
class Umbra extends Config<Umbra, UmbraIN, UmbraOUT> with UmbraConfig {
  Umbra({Key? key}) : super(key: key, fallback: UmbraIN());

  @override
  Umbra createNew() => Umbra();

  factory Umbra.elevated(double size, [int? alpha]) {
    return Umbra()
      ..shadowColor(palette.grey, alpha)
      ..shadowBlur(size)
      ..shadowOffset(size / 5)
      ..shadowSpread(-0.1);
  }
}

class UmbraIN extends ConfigIN<UmbraIN, UmbraOUT> with UmbraConfigIN {
  @override
  UmbraIN createNew() => UmbraIN();

  @override
  UmbraOUT call() => UmbraOUT(this);
}

class UmbraOUT extends ConfigOUT<UmbraOUT, UmbraIN> with UmbraConfigOUT {
  UmbraOUT(UmbraIN iN) : super(iN);
}

/// X3D ~ [X3DConfig] final class
class X3D extends Config<X3D, X3DIN, X3DOUT> with X3DConfig {
  X3D({Key? key}) : super(key: key, fallback: X3DIN());

  @override
  X3D createNew() => X3D();
}

class X3DIN extends ConfigIN<X3DIN, X3DOUT> with X3DConfigIN {
  @override
  X3DIN createNew() => X3DIN();

  @override
  X3DOUT call() => X3DOUT(this);
}

class X3DOUT extends ConfigOUT<X3DOUT, X3DIN> with X3DConfigOUT {
  X3DOUT(X3DIN iN) : super(iN);
}
