part of nueli.core.util;

/// *@operation* Addition
mixin Add<T extends Add<T>> {
  T operator +(T operand);
}

/// *@operation* Multiplication
mixin Multiply<T extends Multiply<T, Operand>, Operand> {
  T operator *(Operand operand);
}

/// *@operation* Division
mixin Divide<T extends Divide<T, Operand>, Operand> {
  T operator /(Operand operand);
}

/// *@operation* AND Operation
mixin AND<T extends AND<T, Operand>, Operand> {
  T operator &(Operand operand);
}

/// *@operation* OR Operation
mixin OR<T extends OR<T, Operand>, Operand> {
  T operator |(Operand operand);
}

/// *@operation* NOT Operation
mixin NOT<T extends NOT<T>> {
  T operator -();
}

extension NumberCalc on num {
  double get modulus => sqrt(this * this);
}

extension PointCalc on Point {
  double operator ~/(Point a) => distanceTo(a);
}

extension DateTimeCalc on DateTime {
  static const daysOfTheWeek = [
    'Monday', 'Tuesday', 'Wednesday', //
    'Thursday', 'Friday', 'Saturday',
    'Sunday' //
  ];

  static const monthsOfTheYear = [
    'January', 'February', 'March', //
    'April', 'May', 'June', 'July', //
    'August', 'September', 'October', //
    'November', 'December' //
  ];

  static const daysOfTheWk = [
    'Mon', 'Tue', 'Wed', 'Thu', //
    'Fri', 'Sat', 'Sun'
  ];

  static const monthsOfTheYr = [
    'Jan', 'Feb', 'Mar', 'Apr', 'May', //
    'Jun', 'Jul', 'Aug', 'Sep', 'Oct', //
    'Nov', 'Dec' //
  ];

  String parse([String? format]) {
    return '$day ${monthsOfTheYr[month - 1]}, $year ${hour % 12} : $minute ${hour > 12 ? 'PM' : 'AM'}';
  }
}
